# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/v1_account_import_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_importAccountV1Wizard(object):
    def setupUi(self, importAccountV1Wizard):
        importAccountV1Wizard.setObjectName("importAccountV1Wizard")
        importAccountV1Wizard.resize(780, 452)
        importAccountV1Wizard.setModal(True)
        self.wizardPage1 = QtWidgets.QWizardPage()
        self.wizardPage1.setObjectName("wizardPage1")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.wizardPage1)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.page1_label = QtWidgets.QLabel(self.wizardPage1)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.page1_label.sizePolicy().hasHeightForWidth())
        self.page1_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setStyleStrategy(QtGui.QFont.PreferAntialias)
        self.page1_label.setFont(font)
        self.page1_label.setWordWrap(True)
        self.page1_label.setObjectName("page1_label")
        self.verticalLayout_6.addWidget(self.page1_label)
        importAccountV1Wizard.addPage(self.wizardPage1)
        self.wizardPage2 = QtWidgets.QWizardPage()
        self.wizardPage2.setObjectName("wizardPage2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.wizardPage2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.page2_label = QtWidgets.QLabel(self.wizardPage2)
        font = QtGui.QFont()
        font.setItalic(False)
        self.page2_label.setFont(font)
        self.page2_label.setAlignment(QtCore.Qt.AlignCenter)
        self.page2_label.setWordWrap(True)
        self.page2_label.setObjectName("page2_label")
        self.verticalLayout.addWidget(self.page2_label)
        self.groupBox = QtWidgets.QGroupBox(self.wizardPage2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.groupBox.setFont(font)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout.setContentsMargins(0, 0, -1, 0)
        self.gridLayout.setHorizontalSpacing(6)
        self.gridLayout.setObjectName("gridLayout")
        self.sourceAddressValueLabel = QtWidgets.QLabel(self.groupBox)
        self.sourceAddressValueLabel.setText("")
        self.sourceAddressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.sourceAddressValueLabel.setObjectName("sourceAddressValueLabel")
        self.gridLayout.addWidget(self.sourceAddressValueLabel, 2, 2, 1, 1)
        self.sourceV1AddressLabel = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceV1AddressLabel.sizePolicy().hasHeightForWidth())
        self.sourceV1AddressLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceV1AddressLabel.setFont(font)
        self.sourceV1AddressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceV1AddressLabel.setObjectName("sourceV1AddressLabel")
        self.gridLayout.addWidget(self.sourceV1AddressLabel, 1, 0, 1, 1)
        self.sourceAddressLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceAddressLabel.setFont(font)
        self.sourceAddressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceAddressLabel.setObjectName("sourceAddressLabel")
        self.gridLayout.addWidget(self.sourceAddressLabel, 2, 0, 1, 1)
        self.sourceBalanceValueLabel = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceBalanceValueLabel.sizePolicy().hasHeightForWidth())
        self.sourceBalanceValueLabel.setSizePolicy(sizePolicy)
        self.sourceBalanceValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);")
        self.sourceBalanceValueLabel.setObjectName("sourceBalanceValueLabel")
        self.gridLayout.addWidget(self.sourceBalanceValueLabel, 3, 2, 1, 1)
        self.sourceIdentityLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceIdentityLabel.setFont(font)
        self.sourceIdentityLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceIdentityLabel.setObjectName("sourceIdentityLabel")
        self.gridLayout.addWidget(self.sourceIdentityLabel, 4, 0, 1, 1)
        self.sourceBalanceLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceBalanceLabel.setFont(font)
        self.sourceBalanceLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceBalanceLabel.setObjectName("sourceBalanceLabel")
        self.gridLayout.addWidget(self.sourceBalanceLabel, 3, 0, 1, 1)
        self.sourceIdentityValueLabel = QtWidgets.QLabel(self.groupBox)
        self.sourceIdentityValueLabel.setText("")
        self.sourceIdentityValueLabel.setObjectName("sourceIdentityValueLabel")
        self.gridLayout.addWidget(self.sourceIdentityValueLabel, 4, 2, 1, 1)
        self.sourceV1AddressValueLabel = QtWidgets.QLabel(self.groupBox)
        self.sourceV1AddressValueLabel.setText("")
        self.sourceV1AddressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.sourceV1AddressValueLabel.setObjectName("sourceV1AddressValueLabel")
        self.gridLayout.addWidget(self.sourceV1AddressValueLabel, 1, 2, 1, 1)
        self.sourceV1AccountLabel = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceV1AccountLabel.setFont(font)
        self.sourceV1AccountLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceV1AccountLabel.setObjectName("sourceV1AccountLabel")
        self.gridLayout.addWidget(self.sourceV1AccountLabel, 0, 0, 1, 1)
        self.sourceV1AccountComboBox = QtWidgets.QComboBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceV1AccountComboBox.sizePolicy().hasHeightForWidth())
        self.sourceV1AccountComboBox.setSizePolicy(sizePolicy)
        self.sourceV1AccountComboBox.setObjectName("sourceV1AccountComboBox")
        self.gridLayout.addWidget(self.sourceV1AccountComboBox, 0, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.sourceErrorLabel = QtWidgets.QLabel(self.groupBox)
        self.sourceErrorLabel.setText("")
        self.sourceErrorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.sourceErrorLabel.setObjectName("sourceErrorLabel")
        self.verticalLayout_2.addWidget(self.sourceErrorLabel)
        self.verticalLayout.addWidget(self.groupBox)
        importAccountV1Wizard.addPage(self.wizardPage2)
        self.wizardPage3 = QtWidgets.QWizardPage()
        self.wizardPage3.setObjectName("wizardPage3")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.wizardPage3)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.page3_label = QtWidgets.QLabel(self.wizardPage3)
        font = QtGui.QFont()
        font.setItalic(False)
        self.page3_label.setFont(font)
        self.page3_label.setAlignment(QtCore.Qt.AlignCenter)
        self.page3_label.setWordWrap(True)
        self.page3_label.setObjectName("page3_label")
        self.verticalLayout_4.addWidget(self.page3_label)
        self.groupBox_2 = QtWidgets.QGroupBox(self.wizardPage3)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.groupBox_2.setFont(font)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.GridLayout_2 = QtWidgets.QGridLayout()
        self.GridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.GridLayout_2.setContentsMargins(-1, 0, -1, 0)
        self.GridLayout_2.setObjectName("GridLayout_2")
        self.destinationAddressLabel = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationAddressLabel.setFont(font)
        self.destinationAddressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.destinationAddressLabel.setObjectName("destinationAddressLabel")
        self.GridLayout_2.addWidget(self.destinationAddressLabel, 3, 0, 1, 1)
        self.destinationAccountLabel = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationAccountLabel.setFont(font)
        self.destinationAccountLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.destinationAccountLabel.setObjectName("destinationAccountLabel")
        self.GridLayout_2.addWidget(self.destinationAccountLabel, 1, 0, 1, 1)
        self.destinationDerivationValueLabel = QtWidgets.QLabel(self.groupBox_2)
        self.destinationDerivationValueLabel.setText("")
        self.destinationDerivationValueLabel.setObjectName("destinationDerivationValueLabel")
        self.GridLayout_2.addWidget(self.destinationDerivationValueLabel, 2, 1, 1, 1)
        self.destinationAccountComboBox = QtWidgets.QComboBox(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.destinationAccountComboBox.sizePolicy().hasHeightForWidth())
        self.destinationAccountComboBox.setSizePolicy(sizePolicy)
        self.destinationAccountComboBox.setObjectName("destinationAccountComboBox")
        self.GridLayout_2.addWidget(self.destinationAccountComboBox, 1, 1, 1, 1)
        self.destinationBalanceLabel = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationBalanceLabel.setFont(font)
        self.destinationBalanceLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.destinationBalanceLabel.setObjectName("destinationBalanceLabel")
        self.GridLayout_2.addWidget(self.destinationBalanceLabel, 4, 0, 1, 1)
        self.destinationBalanceValueLabel = QtWidgets.QLabel(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.destinationBalanceValueLabel.sizePolicy().hasHeightForWidth())
        self.destinationBalanceValueLabel.setSizePolicy(sizePolicy)
        self.destinationBalanceValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);")
        self.destinationBalanceValueLabel.setObjectName("destinationBalanceValueLabel")
        self.GridLayout_2.addWidget(self.destinationBalanceValueLabel, 4, 1, 1, 1)
        self.destinationDerivationLabel = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationDerivationLabel.setFont(font)
        self.destinationDerivationLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.destinationDerivationLabel.setObjectName("destinationDerivationLabel")
        self.GridLayout_2.addWidget(self.destinationDerivationLabel, 2, 0, 1, 1)
        self.destinationAddressValueLabel = QtWidgets.QLabel(self.groupBox_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationAddressValueLabel.setFont(font)
        self.destinationAddressValueLabel.setText("")
        self.destinationAddressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.destinationAddressValueLabel.setObjectName("destinationAddressValueLabel")
        self.GridLayout_2.addWidget(self.destinationAddressValueLabel, 3, 1, 1, 1)
        self.verticalLayout_3.addLayout(self.GridLayout_2)
        self.verticalLayout_4.addWidget(self.groupBox_2)
        self.destinationErrorLabel = QtWidgets.QLabel(self.wizardPage3)
        self.destinationErrorLabel.setText("")
        self.destinationErrorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.destinationErrorLabel.setObjectName("destinationErrorLabel")
        self.verticalLayout_4.addWidget(self.destinationErrorLabel)
        importAccountV1Wizard.addPage(self.wizardPage3)
        self.wizardPage4 = QtWidgets.QWizardPage()
        self.wizardPage4.setObjectName("wizardPage4")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.wizardPage4)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label = QtWidgets.QLabel(self.wizardPage4)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout_5.addWidget(self.label)
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.sourceV1AddressValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceV1AddressValueImportLabel.sizePolicy().hasHeightForWidth())
        self.sourceV1AddressValueImportLabel.setSizePolicy(sizePolicy)
        self.sourceV1AddressValueImportLabel.setText("")
        self.sourceV1AddressValueImportLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.sourceV1AddressValueImportLabel.setObjectName("sourceV1AddressValueImportLabel")
        self.gridLayout_2.addWidget(self.sourceV1AddressValueImportLabel, 1, 1, 1, 1)
        self.sourceV1AddressImportLabel = QtWidgets.QLabel(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceV1AddressImportLabel.sizePolicy().hasHeightForWidth())
        self.sourceV1AddressImportLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceV1AddressImportLabel.setFont(font)
        self.sourceV1AddressImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceV1AddressImportLabel.setObjectName("sourceV1AddressImportLabel")
        self.gridLayout_2.addWidget(self.sourceV1AddressImportLabel, 1, 0, 1, 1)
        self.sourceBalanceValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceBalanceValueImportLabel.sizePolicy().hasHeightForWidth())
        self.sourceBalanceValueImportLabel.setSizePolicy(sizePolicy)
        self.sourceBalanceValueImportLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);")
        self.sourceBalanceValueImportLabel.setObjectName("sourceBalanceValueImportLabel")
        self.gridLayout_2.addWidget(self.sourceBalanceValueImportLabel, 3, 1, 1, 1)
        self.sourceBalanceImportLabel = QtWidgets.QLabel(self.wizardPage4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceBalanceImportLabel.setFont(font)
        self.sourceBalanceImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceBalanceImportLabel.setObjectName("sourceBalanceImportLabel")
        self.gridLayout_2.addWidget(self.sourceBalanceImportLabel, 3, 0, 1, 1)
        self.sourceIdentityImportLabel = QtWidgets.QLabel(self.wizardPage4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceIdentityImportLabel.setFont(font)
        self.sourceIdentityImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceIdentityImportLabel.setObjectName("sourceIdentityImportLabel")
        self.gridLayout_2.addWidget(self.sourceIdentityImportLabel, 4, 0, 1, 1)
        self.sourceIdentityValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceIdentityValueImportLabel.sizePolicy().hasHeightForWidth())
        self.sourceIdentityValueImportLabel.setSizePolicy(sizePolicy)
        self.sourceIdentityValueImportLabel.setText("")
        self.sourceIdentityValueImportLabel.setObjectName("sourceIdentityValueImportLabel")
        self.gridLayout_2.addWidget(self.sourceIdentityValueImportLabel, 4, 1, 1, 1)
        self.sourceAddressImportLabel = QtWidgets.QLabel(self.wizardPage4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceAddressImportLabel.setFont(font)
        self.sourceAddressImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceAddressImportLabel.setObjectName("sourceAddressImportLabel")
        self.gridLayout_2.addWidget(self.sourceAddressImportLabel, 2, 0, 1, 1)
        self.sourceAddressValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceAddressValueImportLabel.sizePolicy().hasHeightForWidth())
        self.sourceAddressValueImportLabel.setSizePolicy(sizePolicy)
        self.sourceAddressValueImportLabel.setText("")
        self.sourceAddressValueImportLabel.setObjectName("sourceAddressValueImportLabel")
        self.gridLayout_2.addWidget(self.sourceAddressValueImportLabel, 2, 1, 1, 1)
        self.sourceNameImportLabel = QtWidgets.QLabel(self.wizardPage4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.sourceNameImportLabel.setFont(font)
        self.sourceNameImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sourceNameImportLabel.setObjectName("sourceNameImportLabel")
        self.gridLayout_2.addWidget(self.sourceNameImportLabel, 0, 0, 1, 1)
        self.sourceNameValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        self.sourceNameValueImportLabel.setText("")
        self.sourceNameValueImportLabel.setObjectName("sourceNameValueImportLabel")
        self.gridLayout_2.addWidget(self.sourceNameValueImportLabel, 0, 1, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout_2)
        self.importButton = QtWidgets.QPushButton(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.importButton.sizePolicy().hasHeightForWidth())
        self.importButton.setSizePolicy(sizePolicy)
        self.importButton.setObjectName("importButton")
        self.verticalLayout_5.addWidget(self.importButton, 0, QtCore.Qt.AlignHCenter)
        self.ImportImagelabel = QtWidgets.QLabel(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ImportImagelabel.sizePolicy().hasHeightForWidth())
        self.ImportImagelabel.setSizePolicy(sizePolicy)
        self.ImportImagelabel.setMaximumSize(QtCore.QSize(64, 64))
        self.ImportImagelabel.setText("")
        self.ImportImagelabel.setPixmap(QtGui.QPixmap(":/images/arrow_down"))
        self.ImportImagelabel.setScaledContents(True)
        self.ImportImagelabel.setAlignment(QtCore.Qt.AlignCenter)
        self.ImportImagelabel.setObjectName("ImportImagelabel")
        self.verticalLayout_5.addWidget(self.ImportImagelabel, 0, QtCore.Qt.AlignHCenter)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.destinationAddressImportLabel = QtWidgets.QLabel(self.wizardPage4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationAddressImportLabel.setFont(font)
        self.destinationAddressImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.destinationAddressImportLabel.setObjectName("destinationAddressImportLabel")
        self.gridLayout_3.addWidget(self.destinationAddressImportLabel, 2, 0, 1, 1)
        self.destinationAddressValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.destinationAddressValueImportLabel.sizePolicy().hasHeightForWidth())
        self.destinationAddressValueImportLabel.setSizePolicy(sizePolicy)
        self.destinationAddressValueImportLabel.setText("")
        self.destinationAddressValueImportLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.destinationAddressValueImportLabel.setObjectName("destinationAddressValueImportLabel")
        self.gridLayout_3.addWidget(self.destinationAddressValueImportLabel, 2, 1, 1, 1)
        self.destinationBalanceValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.destinationBalanceValueImportLabel.sizePolicy().hasHeightForWidth())
        self.destinationBalanceValueImportLabel.setSizePolicy(sizePolicy)
        self.destinationBalanceValueImportLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);")
        self.destinationBalanceValueImportLabel.setObjectName("destinationBalanceValueImportLabel")
        self.gridLayout_3.addWidget(self.destinationBalanceValueImportLabel, 3, 1, 1, 1)
        self.destinationBalanceImportLabel = QtWidgets.QLabel(self.wizardPage4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationBalanceImportLabel.setFont(font)
        self.destinationBalanceImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.destinationBalanceImportLabel.setObjectName("destinationBalanceImportLabel")
        self.gridLayout_3.addWidget(self.destinationBalanceImportLabel, 3, 0, 1, 1)
        self.destinationIdentityImportLabel = QtWidgets.QLabel(self.wizardPage4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationIdentityImportLabel.setFont(font)
        self.destinationIdentityImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.destinationIdentityImportLabel.setObjectName("destinationIdentityImportLabel")
        self.gridLayout_3.addWidget(self.destinationIdentityImportLabel, 4, 0, 1, 1)
        self.destinationIdentityValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        self.destinationIdentityValueImportLabel.setText("")
        self.destinationIdentityValueImportLabel.setObjectName("destinationIdentityValueImportLabel")
        self.gridLayout_3.addWidget(self.destinationIdentityValueImportLabel, 4, 1, 1, 1)
        self.destinationNameImportLabel = QtWidgets.QLabel(self.wizardPage4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.destinationNameImportLabel.setFont(font)
        self.destinationNameImportLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.destinationNameImportLabel.setObjectName("destinationNameImportLabel")
        self.gridLayout_3.addWidget(self.destinationNameImportLabel, 1, 0, 1, 1)
        self.destinationNameValueImportLabel = QtWidgets.QLabel(self.wizardPage4)
        self.destinationNameValueImportLabel.setText("")
        self.destinationNameValueImportLabel.setObjectName("destinationNameValueImportLabel")
        self.gridLayout_3.addWidget(self.destinationNameValueImportLabel, 1, 1, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout_3)
        self.importErrorLabel = QtWidgets.QLabel(self.wizardPage4)
        self.importErrorLabel.setText("")
        self.importErrorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.importErrorLabel.setObjectName("importErrorLabel")
        self.verticalLayout_5.addWidget(self.importErrorLabel)
        importAccountV1Wizard.addPage(self.wizardPage4)

        self.retranslateUi(importAccountV1Wizard)
        QtCore.QMetaObject.connectSlotsByName(importAccountV1Wizard)
        importAccountV1Wizard.setTabOrder(self.destinationAccountComboBox, self.importButton)

    def retranslateUi(self, importAccountV1Wizard):
        #  _translate = QtCore.QCoreApplication.translate
        importAccountV1Wizard.setWindowTitle(_("Wizard to transfer a V1 account into a V2 account"))
        self.page1_label.setText(_("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:22pt;\">This wizard will help you to transfer a V1 account into a V2 account.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:22pt;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:16pt;\">Identity and money will be transfered from the V1 account to the</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:16pt;\">V2 account.</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600; font-style:italic;\">V1 account: </span><span style=\" font-style:italic;\">A V1 account is an Ğ1 account generated by a secret ID and a password (ex: Cesium V1).</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-style:italic;\"><br /></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600; font-style:italic;\">V2 Account:</span><span style=\" font-style:italic;\"> A V2 account is an account generated by a twelve-word restoration code (also called mnemonic).</span></p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.page2_label.setText(_("Please select the V1 source account"))
        self.groupBox.setTitle(_("V1 Source Account"))
        self.sourceV1AddressLabel.setText(_("V1 Address"))
        self.sourceAddressLabel.setText(_("Address"))
        self.sourceBalanceValueLabel.setText(_("?"))
        self.sourceIdentityLabel.setText(_("Identity"))
        self.sourceBalanceLabel.setText(_("Balance"))
        self.sourceV1AccountLabel.setText(_("V1 Account"))
        self.page3_label.setText(_("Please select the V2 destination account"))
        self.groupBox_2.setTitle(_("V2 Destination Account"))
        self.destinationAddressLabel.setText(_("Address"))
        self.destinationAccountLabel.setText(_("V2 account"))
        self.destinationBalanceLabel.setText(_("Balance"))
        self.destinationBalanceValueLabel.setText(_("?"))
        self.destinationDerivationLabel.setText(_("Derivation"))
        self.label.setText(_("Click on the transfer account button and wait for the status message"))
        self.sourceV1AddressImportLabel.setText(_("V1 Address"))
        self.sourceBalanceValueImportLabel.setText(_("?"))
        self.sourceBalanceImportLabel.setText(_("Balance"))
        self.sourceIdentityImportLabel.setText(_("Identity"))
        self.sourceAddressImportLabel.setText(_("Address"))
        self.sourceNameImportLabel.setText(_("Name"))
        self.importButton.setText(_("Transfer account"))
        self.destinationAddressImportLabel.setText(_("Address"))
        self.destinationBalanceValueImportLabel.setText(_("?"))
        self.destinationBalanceImportLabel.setText(_("Balance"))
        self.destinationIdentityImportLabel.setText(_("Identity"))
        self.destinationNameImportLabel.setText(_("Name"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
from typing import TYPE_CHECKING, Dict, List, Optional

from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.node.accounts import (
    NodeAccountsException,
    NodeAccountsInterface,
)

if TYPE_CHECKING:
    from tikka.adapters.network.node.node import NetworkNode


class NodeAccounts(NodeAccountsInterface):
    """
    NodeAccounts class
    """

    def __init__(self, node: "NetworkNode") -> None:
        """
        Use NetworkNodeInterface to request/send smiths information

        :param node: NetworkNodeInterface instance
        :return:
        """
        self.node = node

    def get_balance(self, address: str) -> Optional[int]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeAccountsInterface.get_balance.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeAccountsException(NetworkConnectionError())

        # system.account: FrameSystemAccountInfo
        # {
        #   nonce: 1
        #   consumers: 0
        #   providers: 1
        #   sufficients: 0
        #   data: {
        #     randomId: 0x18a4d...
        #     free: 9,799
        #     reserved: 0
        #     feeFrozen: 0
        #   }
        # }
        try:
            result = self.node.connection.client.query("System", "Account", [address])
        except Exception as exception:
            logging.exception(exception)
            raise NodeAccountsException(exception)

        if result is None:
            balance = None
        else:
            balance = result["data"]["free"] + result["data"]["reserved"]

        return balance

    def get_balances(self, addresses: List[str]) -> Dict[str, Optional[int]]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeAccountsInterface.get_balances.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeAccountsException(NetworkConnectionError())

        storage_functions = []
        for address in addresses:
            storage_functions.append(("System", "Account", [address]))

        try:
            multi_result = self.node.connection.client.query_multi(storage_functions)
        except Exception as exception:
            logging.exception(exception)
            raise NodeAccountsException(exception)

        balances: Dict[str, Optional[int]] = {}
        for index, value_obj in enumerate(multi_result):
            if value_obj is None:
                balances[addresses[index]] = None
            else:
                balances[addresses[index]] = (
                    value_obj["data"]["free"] + value_obj["data"]["reserved"]
                )

        return balances

    def get_unclaimed_ud_balance(self, first_eligible_ud: int) -> int:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeAccountsInterface.get_unclaimed_ud_balance.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeAccountsException(NetworkConnectionError())

        if first_eligible_ud == 0:
            return 0

        try:
            result = self.node.connection.client.query(
                "UniversalDividend", "CurrentUdIndex"
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeAccountsException(exception)

        current_index = result

        try:
            result = self.node.connection.client.query(
                "UniversalDividend", "PastReevals"
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeAccountsException(exception)

        if result is None:
            return 0

        balance = 0
        index = current_index
        for reeval_index, reeval_value in reversed(result):
            if reeval_index <= first_eligible_ud:
                count = index - first_eligible_ud
                balance += count * reeval_value
                break
            else:
                count = index - reeval_index
                balance += count * reeval_value
                index = reeval_index

        return balance



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
import sys
from pathlib import Path
from typing import TYPE_CHECKING, Optional

from PyQt5.QtCore import QMutex, QTimer
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import QApplication, QDialog, QFileDialog, QLineEdit, QWidget

from tikka.domains.application import Application
from tikka.domains.entities.account import Account, AccountCryptoType
from tikka.domains.entities.constants import DATA_PATH, WALLETS_PASSWORD_LENGTH
from tikka.domains.entities.wallet import V1FileWallet
from tikka.libs.keypair import Keypair
from tikka.libs.secret import generate_alphabetic
from tikka.slots.pyqt.entities.constants import (
    ADDRESS_MONOSPACE_FONT_NAME,
    DEBOUNCE_TIME,
    WALLET_LOAD_DEFAULT_DIRECTORY_PREFERENCES_KEY,
)
from tikka.slots.pyqt.entities.worker import AsyncQWorker
from tikka.slots.pyqt.resources.gui.windows.v1_file_import_rc import (
    Ui_V1FileImportDialog,
)

if TYPE_CHECKING:
    pass


class V1FileImportWindow(QDialog, Ui_V1FileImportDialog):
    """
    V1FileImportWindow class
    """

    def __init__(
        self,
        application: Application,
        mutex: QMutex,
        parent: Optional[QWidget] = None,
    ):
        """
        Init V1 import wallet window

        :param application: Application instance
        :param mutex: QMutex instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self.v1_file_wallet: Optional[V1FileWallet] = None
        self._ = self.application.translator.gettext
        self.mutex = mutex

        # set monospace font to address fields
        monospace_font = QFont(ADDRESS_MONOSPACE_FONT_NAME)
        monospace_font.setStyleHint(QFont.Monospace)
        self.addressValueLabel.setFont(monospace_font)
        self.v1AddressValueLabel.setFont(monospace_font)

        # events
        self.showButton.clicked.connect(self._on_show_button_clicked)
        self.browseFilesButton.clicked.connect(self._on_browse_files_button_clicked)
        self.filePasswordLineEdit.textChanged.connect(
            self._on_password_line_edit_changed
        )
        self.buttonBox.rejected.connect(self.close)
        self.buttonBox.accepted.connect(self._on_accepted_button)
        self.changeButton.clicked.connect(self._generate_wallet_password)

        # debounce timer
        self.debounce_timer = QTimer()
        self.debounce_timer.timeout.connect(self._load_v1_file_wallet)
        # Create a QWorker object
        self.network_check_legacy_v1_async_qworker = AsyncQWorker(
            self.network_check_legacy_v1, self.mutex
        )
        self.network_check_legacy_v1_async_qworker.finished.connect(
            self._on_finished_network_check_legacy_v1
        )

        # fill form
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
        self._generate_wallet_password()

    def _generate_wallet_password(self):
        """
        Generate new password for wallet encryption in UI

        :return:
        """
        self.passwordLineEdit.setText(generate_alphabetic(WALLETS_PASSWORD_LENGTH))

    def _on_show_button_clicked(self):
        """
        Triggered when user click on show button

        :return:
        """
        if self.filePasswordLineEdit.echoMode() == QLineEdit.Password:
            self.filePasswordLineEdit.setEchoMode(QLineEdit.Normal)
            self.showButton.setText(self._("Hide"))
        else:
            self.filePasswordLineEdit.setEchoMode(QLineEdit.Password)
            self.showButton.setText(self._("Show"))

    def _on_browse_files_button_clicked(self):
        """
        Triggered when user click on browse files button

        :return:
        """
        # clear path
        self.pathValueLabel.setText("")
        self.filePasswordLineEdit.setText("")

        # open file dialog
        filepath = self.open_file_dialog()
        if filepath is not None:
            # update default dir preference
            self.application.repository.preferences.set(
                WALLET_LOAD_DEFAULT_DIRECTORY_PREFERENCES_KEY,
                str(Path(filepath).expanduser().absolute().parent),
            )
            self.pathValueLabel.setText(filepath)

    def _load_v1_file_wallet(self):
        """
        Load V1 file wallet

        :return:
        """
        # stop debounce_timer to avoid infinite loop
        if self.debounce_timer.isActive():
            self.debounce_timer.stop()

        password = self.filePasswordLineEdit.text().strip()
        try:
            self.v1_file_wallet = self.application.repository.v1_file_wallets.load(
                self.pathValueLabel.text(), password
            )
        except Exception:
            self.errorLabel.setText(self._("Unable to decrypt file!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return

        if not self.v1_file_wallet.signing_key:
            self.errorLabel.setText(self._("File password is not valid!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return

        # wallet decrypted
        keypair = Keypair.create_from_seed(
            seed_hex=self.v1_file_wallet.signing_key.seed.hex(),
            ss58_format=self.application.currencies.get_current().ss58_format,
            crypto_type=AccountCryptoType.ED25519,
        )
        self.v1AddressValueLabel.setText(
            Account(keypair.ss58_address).get_v1_address(
                self.application.currencies.get_current().ss58_format
            )
        )
        self.addressValueLabel.setText(keypair.ss58_address)
        if self.application.accounts.get_by_address(keypair.ss58_address) is not None:
            self.errorLabel.setText(self._("Account already exists!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
        else:
            self.errorLabel.setText("")
            self.network_check_legacy_v1_async_qworker.start()

    def open_file_dialog(self) -> Optional[str]:
        """
        Open file dialog and return the selected filepath or None

        :return:
        """
        default_dir = self.application.repository.preferences.get(
            WALLET_LOAD_DEFAULT_DIRECTORY_PREFERENCES_KEY
        )
        if default_dir is not None:
            default_dir = str(Path(default_dir).expanduser().absolute())
        else:
            default_dir = ""

        result = QFileDialog.getOpenFileName(
            self, self._("Choose wallet file"), default_dir, "EWIF Files (*.dunikey)"
        )
        if result[0] == "":
            return None

        return result[0]

    def _on_password_line_edit_changed(self):
        """
        Triggered when text is changed in the password field

        :return:
        """
        if self.debounce_timer.isActive():
            self.debounce_timer.stop()
        self.debounce_timer.start(DEBOUNCE_TIME)

    def network_check_legacy_v1(self):
        """
        Check if account is really a known legacy v1 account

        :return:
        """
        address = self.addressValueLabel.text().strip()
        try:
            self.is_legacy_v1 = self.application.accounts.network_is_legacy_v1(address)
        except Exception as exception:
            self.errorLabel.setText(self._(str(exception)))
            logging.exception(exception)

    def _on_finished_network_check_legacy_v1(self):
        """
        Triggered when async request network_check_legacy_v1 is finished

        :return:
        """
        if self.is_legacy_v1 is not None:
            if not self.is_legacy_v1:
                self.errorLabel.setText(self._("V1 account unknown!"))
                self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            else:
                self.errorLabel.setText("")
                self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)

    def _on_accepted_button(self):
        """
        Triggered when user click on cancel button

        :return:
        """
        if self.v1_file_wallet is None:
            return

        name = self.nameLineEdit.text().strip()
        password = self.passwordLineEdit.text()

        account = self.application.accounts.create_new_root_account_v1_from_seed(
            self.v1_file_wallet.signing_key.seed.hex(), name, password
        )
        if account is not None:
            # set account from file flag
            account.file_import = True
            self.application.accounts.update(account)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    V1FileImportWindow(application_, QMutex()).exec_()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Dict, List

from gql import gql

from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.indexer.identities import (
    IndexerIdentitiesException,
    IndexerIdentitiesInterface,
)


class IndexerIdentities(IndexerIdentitiesInterface):
    """
    IndexerIdentities class
    """

    def get_identity_name(self, identity_index: int) -> str:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            IndexerIdentitiesInterface.get_identity_name.__doc__
        )
        if (
            not self.indexer.connection.is_connected()
            or self.indexer.connection.client is None
        ):
            raise IndexerIdentitiesException(NetworkConnectionError())

        query = gql(
            """
                query {
                    identity(
                        where: {
                            index: {_eq: """
            + str(identity_index)
            + """ } 
                        } 
                        ) {
                            name
                    }
                }
                """
        )
        try:
            result = self.indexer.connection.client.execute(query)
        except Exception as exception:
            raise IndexerIdentitiesException(exception)

        #
        #    {
        #         "identity": [
        #             {
        #                 "name": "vit"
        #             }
        #         ]
        #     }
        #
        return result["identity"][0]["name"]

    def get_identity_names(self, index_list: List[int]) -> Dict[int, str]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            IndexerIdentitiesInterface.get_identity_names.__doc__
        )
        if (
            not self.indexer.connection.is_connected()
            or self.indexer.connection.client is None
        ):
            raise IndexerIdentitiesException(NetworkConnectionError())

        query = gql(
            """
                query GetNamesByIndex($indices: [Int!]!) {
                    identity(where: { index: { _in: $indices } }) {
                        index
                        name
                    }
                }
                """
        )
        variables = {"indices": index_list}
        try:
            result = self.indexer.connection.client.execute(
                query, variable_values=variables
            )
        except Exception as exception:
            raise IndexerIdentitiesException(exception)

        #
        #    {
        #         "identity": [
        #             {
        #                 "index": 65,
        #                 "name": "jean"
        #             },
        #             {
        #                 "index": 112,
        #                 "name": "marc"
        #             },
        #         ]
        #     }
        #
        names = {}
        for row in result["identity"]:
            names[row["index"]] = row["name"]
        return names



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
from typing import TYPE_CHECKING, Optional

from tikka.adapters.network.node.substrate_client import ExtrinsicReceipt
from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.node.transfers import (
    NodeTransfersException,
    NodeTransfersInterface,
)
from tikka.libs.keypair import Keypair

if TYPE_CHECKING:
    from tikka.adapters.network.node.node import NetworkNode


class NodeTransfers(NodeTransfersInterface):
    """
    NodeTransfers class
    """

    def __init__(self, node: "NetworkNode") -> None:
        """
        Use NetworkNodeInterface to request/send smiths information

        :param node: NetworkNodeInterface instance
        :return:
        """
        self.node = node

    def send(
        self, sender_keypair: Keypair, recipient_address: str, amount: int
    ) -> Optional[ExtrinsicReceipt]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeTransfersInterface.send.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeTransfersException(NetworkConnectionError())

        params = {"" "dest": recipient_address, "value": amount}
        try:
            call = self.node.connection.client.compose_call(
                call_module="Balances",
                call_function="transfer_keep_alive",
                call_params=params,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        try:
            extrinsic = self.node.connection.client.create_signed_extrinsic(
                call=call, keypair=sender_keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        try:
            # fixme: code stuck infinitely if no blocks are created on blockchain
            #       should have a timeout option
            receipt = self.node.connection.client.submit_extrinsic(
                extrinsic, wait_for_inclusion=True, wait_for_finalization=False
            )
            logging.debug(
                "Extrinsic '%s' sent and included in block '%s'",
                receipt.extrinsic_hash,
                receipt.block_hash,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        if not receipt.is_success:
            logging.error(receipt.error_message)
            message = (
                receipt.error_message.docs[0]
                if receipt.error_message
                else "transfers.send error"
            )
            raise NodeTransfersException(message)

        return receipt

    def send_with_comment(
        self, sender_keypair: Keypair, recipient_address: str, amount: int, comment: str
    ) -> Optional[ExtrinsicReceipt]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeTransfersInterface.send_with_comment.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeTransfersException(NetworkConnectionError())

        transfer_params = {"dest": recipient_address, "value": amount}
        try:
            transfer_call = self.node.connection.client.compose_call(
                call_module="Balances",
                call_function="transfer_keep_alive",
                call_params=transfer_params,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        remark_params = {"remark": comment}
        try:
            remark_call = self.node.connection.client.compose_call(
                call_module="System",
                call_function="remark_with_event",
                call_params=remark_params,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        try:
            call = self.node.connection.client.compose_call(
                call_module="Utility",
                call_function="batch",
                call_params={"calls": [transfer_call, remark_call]},
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)
        try:
            extrinsic = self.node.connection.client.create_signed_extrinsic(
                call=call, keypair=sender_keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        try:
            # fixme: code stuck infinitely if no blocks are created on blockchain
            #       should have a timeout option
            receipt = self.node.connection.client.submit_extrinsic(
                extrinsic, wait_for_inclusion=True, wait_for_finalization=False
            )
            logging.debug(
                "Extrinsic '%s' sent and included in block '%s'",
                receipt.extrinsic_hash,
                receipt.block_hash,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        if not receipt.is_success:
            logging.error(receipt.error_message)
            message = (
                receipt.error_message.docs[0]
                if receipt.error_message
                else "transfers.send_with_comment error"
            )
            raise NodeTransfersException(message)
        return receipt

    def fees(
        self, sender_keypair: Keypair, recipient_address: str, amount: int
    ) -> Optional[int]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeTransfersInterface.fees.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeTransfersException(NetworkConnectionError())

        params = {"dest": recipient_address, "value": amount}
        try:
            call = self.node.connection.client.compose_call(
                call_module="Balances",
                call_function="transfer_keep_alive",
                call_params=params,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        try:
            # Get payment info
            payment_info = self.node.connection.client.get_payment_info(
                call=call, keypair=sender_keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        if payment_info is None:
            return None

        return payment_info["partialFee"]

    def transfer_all(
        self, sender_keypair: Keypair, recipient_address: str, keep_alive: bool = False
    ) -> Optional[ExtrinsicReceipt]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeTransfersInterface.transfer_all.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeTransfersException(NetworkConnectionError())

        params = {"dest": recipient_address, "keep_alive": keep_alive}
        try:
            call = self.node.connection.client.compose_call(
                call_module="Balances",
                call_function="transfer_all",
                call_params=params,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        try:
            extrinsic = self.node.connection.client.create_signed_extrinsic(
                call=call, keypair=sender_keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        try:
            # fixme: code stuck infinitely if no blocks are created on blockchain
            #       should have a timeout option
            receipt = self.node.connection.client.submit_extrinsic(
                extrinsic, wait_for_inclusion=True
            )
            logging.debug(
                "Extrinsic '%s' sent and included in block '%s'",
                receipt.extrinsic_hash,
                receipt.block_hash,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTransfersException(exception)

        if not receipt.is_success:
            logging.error(receipt.error_message)
            message = (
                receipt.error_message.docs[0]
                if receipt.error_message
                else "transfers.transfer_all error"
            )
            raise NodeTransfersException(message)

        return receipt



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
import logging
from queue import Queue
from threading import Thread
from time import sleep

from substrateinterface import SubstrateInterface


class ThreadWorker(Thread):

    queue: Queue = Queue()

    def __init__(self, *args, **kwargs):
        """
        Init a SubstrateInterface client adapter instance as a thread

        :param args: Positional arguments
        :param kwargs: Keywords arguments
        """
        super().__init__(*args, **kwargs)

    def run(self):
        """
        Started asynchronously with Thread.start()

        :return:
        """
        while True:
            # print("loop...")
            call, args, result = self.queue.get()
            result_ = dict()
            # print(call, args, result)
            if call == "--close--":
                logging.debug("Close queue thread on substrate_interface")
                break

            try:
                # logging.debug(f"threadsafe call to rpc method {method}")
                result_ = call(*args)
            except Exception as exception:
                logging.error(args)
                # logging.exception(exception)
                result.put(exception)
            # print(call.__name__, " put result ", result_)
            result.put(result_)
            # print("reloop...")

        logging.debug("SubstrateInterface connection closed and thread terminated.")

    def close(self):
        """
        Close connection

        :return:
        """
        # Closing the connection
        self.queue.put(("--close--", (), None))


class ThreadSafeSubstrateInterface(SubstrateInterface):
    """
    Override substrate_interface client class with a queue to be thread safe

    """

    # keep alive ping interval in seconds
    KEEP_ALIVE_INTERVAL = 30

    def __init__(self, *args, **kwargs):
        """
        Init a SubstrateInterface client adapter instance as a thread

        :param args: Positional arguments
        :param kwargs: Keywords arguments
        """
        # create and start thread before calling parent init (which makes a rpc_request!)
        self.thread = ThreadWorker()
        self.thread.start()
        self.subscription_pending = False

        try:
            super().__init__(*args, **kwargs)
        except ConnectionRefusedError as exception:
            self.thread.close()
            logging.exception(exception)
        else:
            self.keep_alive_flag = True
            self.keep_alive_interval_time = self.KEEP_ALIVE_INTERVAL
            # start keep alive thread
            self.keep_alive_thread = Thread(target=self.keep_alive_ping)
            self.keep_alive_thread.start()

    def keep_alive_ping(self):
        """
        Send a ping message to server to keep alive the websocket connection

        :return:
        """
        while self.keep_alive_flag is True:
            sleep(self.KEEP_ALIVE_INTERVAL / 100)
            self.keep_alive_interval_time -= self.KEEP_ALIVE_INTERVAL / 100
            if self.keep_alive_flag is False:
                logging.debug("Stop websocket keep alive thread")
                break
            if self.keep_alive_interval_time <= 0:
                self.keep_alive_interval_time = self.KEEP_ALIVE_INTERVAL
                result: Queue = Queue()
                logging.debug("Send websocket keep alive ping...")
                self.thread.queue.put(
                    (self.websocket.send, (json.dumps({"ping": 1}),), result)
                )
                logging.debug("Websocket keep alive pong %s received", result.get())

    def rpc_request(self, method, params, result_handler=None) -> dict:
        """
        Override rpc_request method to use threadsafe queue

        :param method: Name of the RPC method
        :param params: Params of the RPC method
        :param result_handler: Optional variable to receive results, default to None
        :return:
        """
        # print("queued rpc_request call !!")
        # if subscription...
        if result_handler is not None:
            # if no subscription in queue...
            if self.subscription_pending is False:
                # set flag
                self.subscription_pending = True
        else:
            if self.subscription_pending is True:
                return super().rpc_request(method, params, result_handler)

        # reset keep alive timer
        self.keep_alive_interval_time = self.KEEP_ALIVE_INTERVAL

        result: Queue = Queue()
        self.thread.queue.put(
            (super().rpc_request, (method, params, result_handler), result)
        )
        # print(self.thread.queue.get())
        # print('done calling %s' % method)
        return_ = result.get()
        # if request was with subscription...
        if self.subscription_pending is True:
            # end of subscription
            self.subscription_pending = False
        if isinstance(return_, Exception):
            raise return_
        return return_

    def close(self):
        logging.debug("Close RPC connection thread")
        self.keep_alive_flag = False
        self.thread.close()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
import logging
from datetime import datetime, timedelta
from typing import TYPE_CHECKING, List

from tikka.domains.entities.technical_committee import (
    TechnicalCommitteeCall,
    TechnicalCommitteeMember,
    TechnicalCommitteeProposal,
    TechnicalCommitteeVoting,
)
from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.node.technical_committee import (
    NodeTechnicalCommitteeException,
    NodeTechnicalCommitteeInterface,
)
from tikka.libs.keypair import Keypair

if TYPE_CHECKING:
    from tikka.adapters.network.node.node import NetworkNode


class NodeTechnicalCommittee(NodeTechnicalCommitteeInterface):
    """
    NodeTechnicalCommittee class
    """

    def __init__(self, node: "NetworkNode") -> None:
        """
        Use NetworkNodeInterface to request/send smiths information

        :param node: NetworkNodeInterface instance
        :return:
        """
        self.node = node

    def members(self) -> List[TechnicalCommitteeMember]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeTechnicalCommitteeInterface.members.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeTechnicalCommitteeException(NetworkConnectionError())

        try:
            member_addresses = self.node.connection.client.query(
                "TechnicalCommittee", "Members"
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTechnicalCommitteeException(exception)

        members = []
        if member_addresses is not None:
            try:
                identities = self.node.identities.get_identities(member_addresses)
            except Exception as exception:
                logging.exception(exception)
                raise NodeTechnicalCommitteeException(exception)

            for address, identity in identities.items():
                members.append(
                    TechnicalCommitteeMember(
                        address=address,
                        identity_index=identity.index if identity else None,  # type: ignore
                    )
                )

        return members

    def proposals(self) -> List[TechnicalCommitteeProposal]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeTechnicalCommitteeInterface.proposals.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeTechnicalCommitteeException(NetworkConnectionError())

        try:
            result = self.node.connection.client.query(
                "TechnicalCommittee", "Proposals"
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTechnicalCommitteeException(exception)

        proposal_hash_256_list = result or []

        try:
            current_block = self.node.connection.client.rpc_request(  # type: ignore
                "system_syncState", []
            ).get("result")["currentBlock"]
        except Exception as exception:
            logging.exception(exception)
            raise NodeTechnicalCommitteeException(exception)

        try:
            result = self.node.connection.client.get_constant(
                "Babe", "ExpectedBlockTime"
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTechnicalCommitteeException(exception)

        block_duration_in_ms = result
        current_time = datetime.now()

        proposals = []
        for proposal_hash_256 in proposal_hash_256_list:

            # fetch Voting
            try:
                result = self.node.connection.client.query(
                    "TechnicalCommittee", "Voting", [proposal_hash_256]
                )
            except Exception as exception:
                logging.exception(exception)
                raise NodeTechnicalCommitteeException(exception)

            block_diff = result["end"] - current_block
            if block_diff < 0:
                block_time = current_time - timedelta(
                    milliseconds=abs(block_diff) * block_duration_in_ms
                )
            else:
                block_time = current_time + timedelta(
                    milliseconds=abs(block_diff) * block_duration_in_ms
                )

            voting = TechnicalCommitteeVoting(
                result["index"],
                result["threshold"],
                result["ayes"],
                result["nays"],
                block_time,
            )

            # fetch Call
            try:
                result = self.node.connection.client.query(
                    "TechnicalCommittee", "ProposalOf", [proposal_hash_256]
                )
            except Exception as exception:
                logging.exception(exception)
                raise NodeTechnicalCommitteeException(exception)

            call = TechnicalCommitteeCall(
                result["call_index"],
                result["call_hash"],
                result["call_module"],
                result["call_function"],
                json.dumps(result["call_args"]),
            )
            proposal = TechnicalCommitteeProposal(proposal_hash_256, call, voting)

            proposals.append(proposal)

        return proposals

    def vote(
        self, keypair: Keypair, proposal: TechnicalCommitteeProposal, vote: bool
    ) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeTechnicalCommitteeInterface.proposals.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeTechnicalCommitteeException(NetworkConnectionError())

        params = {
            "proposal": proposal.hash,
            "index": proposal.voting.index,
            "approve": vote,
        }

        try:
            call = self.node.connection.client.compose_call(
                call_module="TechnicalCommittee",
                call_function="vote",
                call_params=params,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTechnicalCommitteeException(exception)

        try:
            extrinsic = self.node.connection.client.create_signed_extrinsic(
                call=call, keypair=keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeTechnicalCommitteeException(exception)

        # fixme: code stuck infinitely if no blocks are created on blockchain
        #       should have a timeout option
        result = self.node.connection.client.submit_extrinsic(
            extrinsic, wait_for_inclusion=True
        )
        logging.debug(
            "Extrinsic '%s' sent and included in block '%s'",
            result.extrinsic_hash,
            result.block_hash,
        )

        if not result.is_success:
            logging.error(result.error_message)
            message = (
                result.error_message.docs[0]
                if result.error_message
                else "technical_committee.vote error"
            )
            raise NodeTechnicalCommitteeException(message)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/v1_wallet_restore_from_file.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_V1WalletRestoreFromFileDialog(object):
    def setupUi(self, V1WalletRestoreFromFileDialog):
        V1WalletRestoreFromFileDialog.setObjectName("V1WalletRestoreFromFileDialog")
        V1WalletRestoreFromFileDialog.resize(591, 538)
        V1WalletRestoreFromFileDialog.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.verticalLayout = QtWidgets.QVBoxLayout(V1WalletRestoreFromFileDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.imageLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        self.imageLabel.setMaximumSize(QtCore.QSize(200, 200))
        self.imageLabel.setText("")
        self.imageLabel.setPixmap(QtGui.QPixmap(":/images/keys"))
        self.imageLabel.setScaledContents(True)
        self.imageLabel.setObjectName("imageLabel")
        self.verticalLayout.addWidget(self.imageLabel, 0, QtCore.Qt.AlignHCenter)
        self.browseFilesButton = QtWidgets.QPushButton(V1WalletRestoreFromFileDialog)
        self.browseFilesButton.setAutoExclusive(False)
        self.browseFilesButton.setObjectName("browseFilesButton")
        self.verticalLayout.addWidget(self.browseFilesButton, 0, QtCore.Qt.AlignHCenter)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.showButton = QtWidgets.QPushButton(V1WalletRestoreFromFileDialog)
        self.showButton.setObjectName("showButton")
        self.gridLayout.addWidget(self.showButton, 1, 2, 1, 1)
        self.filePasswordLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.filePasswordLabel.setFont(font)
        self.filePasswordLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.filePasswordLabel.setObjectName("filePasswordLabel")
        self.gridLayout.addWidget(self.filePasswordLabel, 1, 0, 1, 1)
        self.v1AddressLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.v1AddressLabel.setFont(font)
        self.v1AddressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.v1AddressLabel.setObjectName("v1AddressLabel")
        self.gridLayout.addWidget(self.v1AddressLabel, 2, 0, 1, 1)
        self.v1AddressValueLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        self.v1AddressValueLabel.setText("")
        self.v1AddressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.v1AddressValueLabel.setObjectName("v1AddressValueLabel")
        self.gridLayout.addWidget(self.v1AddressValueLabel, 2, 1, 1, 1)
        self.filePasswordLineEdit = QtWidgets.QLineEdit(V1WalletRestoreFromFileDialog)
        self.filePasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.filePasswordLineEdit.setObjectName("filePasswordLineEdit")
        self.gridLayout.addWidget(self.filePasswordLineEdit, 1, 1, 1, 1)
        self.pathLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.pathLabel.setFont(font)
        self.pathLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.pathLabel.setObjectName("pathLabel")
        self.gridLayout.addWidget(self.pathLabel, 0, 0, 1, 1)
        self.pathValueLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        self.pathValueLabel.setText("")
        self.pathValueLabel.setObjectName("pathValueLabel")
        self.gridLayout.addWidget(self.pathValueLabel, 0, 1, 1, 1)
        self.addressLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.addressLabel.setObjectName("addressLabel")
        self.gridLayout.addWidget(self.addressLabel, 3, 0, 1, 1)
        self.addressValueLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.gridLayout.addWidget(self.addressValueLabel, 3, 1, 1, 1)
        self.nameLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.nameLabel.setFont(font)
        self.nameLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.nameLabel.setObjectName("nameLabel")
        self.gridLayout.addWidget(self.nameLabel, 4, 0, 1, 1)
        self.nameValueLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        self.nameValueLabel.setText("")
        self.nameValueLabel.setObjectName("nameValueLabel")
        self.gridLayout.addWidget(self.nameValueLabel, 4, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.errorLabel = QtWidgets.QLabel(V1WalletRestoreFromFileDialog)
        self.errorLabel.setStyleSheet("color: rgb(255, 0, 0)")
        self.errorLabel.setText("")
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel, 0, QtCore.Qt.AlignHCenter)
        self.storedPasswordFrame = QtWidgets.QFrame(V1WalletRestoreFromFileDialog)
        self.storedPasswordFrame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.storedPasswordFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.storedPasswordFrame.setObjectName("storedPasswordFrame")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.storedPasswordFrame)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.storedPasswordLabel = QtWidgets.QLabel(self.storedPasswordFrame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.storedPasswordLabel.setFont(font)
        self.storedPasswordLabel.setObjectName("storedPasswordLabel")
        self.horizontalLayout_2.addWidget(self.storedPasswordLabel, 0, QtCore.Qt.AlignRight)
        self.storedpasswordLineEdit = QtWidgets.QLineEdit(self.storedPasswordFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.storedpasswordLineEdit.sizePolicy().hasHeightForWidth())
        self.storedpasswordLineEdit.setSizePolicy(sizePolicy)
        self.storedpasswordLineEdit.setMaxLength(20)
        self.storedpasswordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.storedpasswordLineEdit.setAlignment(QtCore.Qt.AlignCenter)
        self.storedpasswordLineEdit.setReadOnly(True)
        self.storedpasswordLineEdit.setObjectName("storedpasswordLineEdit")
        self.horizontalLayout_2.addWidget(self.storedpasswordLineEdit, 0, QtCore.Qt.AlignHCenter)
        self.storedPasswordHintLabel = QtWidgets.QLabel(self.storedPasswordFrame)
        font = QtGui.QFont()
        font.setItalic(True)
        self.storedPasswordHintLabel.setFont(font)
        self.storedPasswordHintLabel.setObjectName("storedPasswordHintLabel")
        self.horizontalLayout_2.addWidget(self.storedPasswordHintLabel, 0, QtCore.Qt.AlignLeft)
        self.verticalLayout.addWidget(self.storedPasswordFrame)
        self.passwordFrame = QtWidgets.QFrame(V1WalletRestoreFromFileDialog)
        self.passwordFrame.setObjectName("passwordFrame")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.passwordFrame)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.passwordLabel = QtWidgets.QLabel(self.passwordFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLabel.sizePolicy().hasHeightForWidth())
        self.passwordLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.passwordLabel.setFont(font)
        self.passwordLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.passwordLabel.setObjectName("passwordLabel")
        self.horizontalLayout.addWidget(self.passwordLabel, 0, QtCore.Qt.AlignRight)
        self.passwordLineEdit = QtWidgets.QLineEdit(self.passwordFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLineEdit.sizePolicy().hasHeightForWidth())
        self.passwordLineEdit.setSizePolicy(sizePolicy)
        self.passwordLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.passwordLineEdit.setAlignment(QtCore.Qt.AlignCenter)
        self.passwordLineEdit.setReadOnly(True)
        self.passwordLineEdit.setObjectName("passwordLineEdit")
        self.horizontalLayout.addWidget(self.passwordLineEdit, 0, QtCore.Qt.AlignHCenter)
        self.changeButton = QtWidgets.QPushButton(self.passwordFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.changeButton.sizePolicy().hasHeightForWidth())
        self.changeButton.setSizePolicy(sizePolicy)
        self.changeButton.setObjectName("changeButton")
        self.horizontalLayout.addWidget(self.changeButton, 0, QtCore.Qt.AlignLeft)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.verticalLayout.addWidget(self.passwordFrame)
        self.buttonBox = QtWidgets.QDialogButtonBox(V1WalletRestoreFromFileDialog)
        self.buttonBox.setEnabled(True)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(V1WalletRestoreFromFileDialog)
        self.buttonBox.rejected.connect(V1WalletRestoreFromFileDialog.reject) # type: ignore
        self.buttonBox.accepted.connect(V1WalletRestoreFromFileDialog.accept) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(V1WalletRestoreFromFileDialog)
        V1WalletRestoreFromFileDialog.setTabOrder(self.browseFilesButton, self.filePasswordLineEdit)
        V1WalletRestoreFromFileDialog.setTabOrder(self.filePasswordLineEdit, self.storedpasswordLineEdit)
        V1WalletRestoreFromFileDialog.setTabOrder(self.storedpasswordLineEdit, self.passwordLineEdit)
        V1WalletRestoreFromFileDialog.setTabOrder(self.passwordLineEdit, self.changeButton)
        V1WalletRestoreFromFileDialog.setTabOrder(self.changeButton, self.showButton)

    def retranslateUi(self, V1WalletRestoreFromFileDialog):
        #  _translate = QtCore.QCoreApplication.translate
        V1WalletRestoreFromFileDialog.setWindowTitle(_("Restore V1 wallet from file"))
        self.browseFilesButton.setText(_("Browse files..."))
        self.showButton.setText(_("Show"))
        self.filePasswordLabel.setText(_("File Password"))
        self.v1AddressLabel.setText(_("V1 address"))
        self.pathLabel.setText(_("Path"))
        self.addressLabel.setText(_("Address"))
        self.nameLabel.setText(_("Name"))
        self.storedPasswordLabel.setText(_("Password"))
        self.storedPasswordHintLabel.setText(_("Wallet will be locked with the current password"))
        self.passwordLabel.setText(_("Password"))
        self.changeButton.setText(_("Change"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Any

from tikka.interfaces.adapters.repository.config import ConfigRepositoryInterface
from tikka.interfaces.domains.config import ConfigInterface


class Config(ConfigInterface):
    """
    Config domain class
    """

    def __init__(self, repository: ConfigRepositoryInterface):
        """
        Init Config instance

        :param repository: ConfigRepositoryInterface instance
        """
        self.repository = repository
        self.data = self.repository.load()

    def set(self, name: str, value: Any):
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            ConfigInterface.set.__doc__
        )
        self.data[name] = value
        self.repository.save(self.data)

    def get(self, name: str):
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            ConfigInterface.get.__doc__
        )
        return self.data[name]



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import libnacl


def is_valid_ed25519(public_key: bytes) -> bool:
    """
    Check if a public key is valid on the Ed25519 curve.
    Equivalent to isValidEd25519 in TypeScript which uses isTorsionFree().

    This implementation performs the following checks:
    1. Correct length (32 bytes)
    2. Point is on the Ed25519 curve and has the correct order (is torsion-free)

    :param public_key: Public key as bytes

    :return:
    """
    if len(public_key) != 32:
        return False

    # Check for the identity point (all zeros) - not a valid public key
    if public_key == bytes([0] * 32):
        return False

    # Check for the point at infinity or other special encodings
    if (
        public_key[0] == 0x01
        and all(b == 0 for b in public_key[1:-1])
        and public_key[-1] == 0xFE
    ):
        return False

    # Now perform more rigorous checks
    return bool(libnacl.nacl.crypto_core_ed25519_is_valid_point(public_key))


def is_valid_sr25519(public_key: bytes) -> bool:
    """
    Check if a public key is valid on the Sr25519 curve.
    Equivalent to isValidSr25519 in TypeScript.

    Uses pysodium for Ristretto point validation, which is the basis for Sr25519.
    Sr25519 is based on Ristretto255, a specific encoding of Curve25519 points.

    :param public_key: Public key as bytes

    :return:
    """
    # import pysodium

    if len(public_key) != 32:
        return False

    # Sr25519 uses Ristretto points, which are a specific encoding of Curve25519 points
    # We can check if the point is valid by verifying it's a valid Ristretto point

    # we can use crypto_core_ristretto255_is_valid_point to check
    # if the provided bytes represent a valid Ristretto point
    is_valid = libnacl.nacl.crypto_core_ristretto255_is_valid_point(public_key)

    # Additionally, we should check that the point is not the identity element
    # and has the correct order (not a small subgroup element)
    if is_valid:
        # The identity element in Ristretto is all zeros
        if all(b == 0 for b in public_key):
            return False

        # In a complete implementation, we would also check for small subgroup elements
        # but this requires more complex operations beyond the scope of this example

        return True
    return False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/account_table.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountTableWidget(object):
    def setupUi(self, AccountTableWidget):
        AccountTableWidget.setObjectName("AccountTableWidget")
        AccountTableWidget.resize(684, 329)
        AccountTableWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.verticalLayout = QtWidgets.QVBoxLayout(AccountTableWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.refreshButton = QtWidgets.QPushButton(AccountTableWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon)
        self.refreshButton.setObjectName("refreshButton")
        self.verticalLayout.addWidget(self.refreshButton, 0, QtCore.Qt.AlignRight)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.errorLabel = QtWidgets.QLabel(AccountTableWidget)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.horizontalLayout_3.addWidget(self.errorLabel)
        self.totalBalanceValueLabel = QtWidgets.QLabel(AccountTableWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.totalBalanceValueLabel.sizePolicy().hasHeightForWidth())
        self.totalBalanceValueLabel.setSizePolicy(sizePolicy)
        self.totalBalanceValueLabel.setText("")
        self.totalBalanceValueLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.totalBalanceValueLabel.setObjectName("totalBalanceValueLabel")
        self.horizontalLayout_3.addWidget(self.totalBalanceValueLabel)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.filtersGroupBox = QtWidgets.QGroupBox(AccountTableWidget)
        self.filtersGroupBox.setAutoFillBackground(False)
        self.filtersGroupBox.setStyleSheet("")
        self.filtersGroupBox.setFlat(False)
        self.filtersGroupBox.setObjectName("filtersGroupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.filtersGroupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.categoryLabel = QtWidgets.QLabel(self.filtersGroupBox)
        self.categoryLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.categoryLabel.setObjectName("categoryLabel")
        self.horizontalLayout.addWidget(self.categoryLabel)
        self.categoryComboBox = QtWidgets.QComboBox(self.filtersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.categoryComboBox.sizePolicy().hasHeightForWidth())
        self.categoryComboBox.setSizePolicy(sizePolicy)
        self.categoryComboBox.setObjectName("categoryComboBox")
        self.horizontalLayout.addWidget(self.categoryComboBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.walletLabel = QtWidgets.QLabel(self.filtersGroupBox)
        self.walletLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.walletLabel.setObjectName("walletLabel")
        self.horizontalLayout_2.addWidget(self.walletLabel)
        self.walletComboBox = QtWidgets.QComboBox(self.filtersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.walletComboBox.sizePolicy().hasHeightForWidth())
        self.walletComboBox.setSizePolicy(sizePolicy)
        self.walletComboBox.setObjectName("walletComboBox")
        self.horizontalLayout_2.addWidget(self.walletComboBox)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addWidget(self.filtersGroupBox)
        self.tableView = QtWidgets.QTableView(AccountTableWidget)
        self.tableView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tableView.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.tableView.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustIgnored)
        self.tableView.setAutoScroll(True)
        self.tableView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableView.setAlternatingRowColors(True)
        self.tableView.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableView.setShowGrid(False)
        self.tableView.setSortingEnabled(False)
        self.tableView.setObjectName("tableView")
        self.tableView.horizontalHeader().setMinimumSectionSize(50)
        self.tableView.horizontalHeader().setStretchLastSection(True)
        self.tableView.verticalHeader().setVisible(False)
        self.tableView.verticalHeader().setCascadingSectionResizes(True)
        self.tableView.verticalHeader().setDefaultSectionSize(0)
        self.verticalLayout.addWidget(self.tableView)

        self.retranslateUi(AccountTableWidget)
        QtCore.QMetaObject.connectSlotsByName(AccountTableWidget)
        AccountTableWidget.setTabOrder(self.categoryComboBox, self.walletComboBox)
        AccountTableWidget.setTabOrder(self.walletComboBox, self.tableView)

    def retranslateUi(self, AccountTableWidget):
        #  _translate = QtCore.QCoreApplication.translate
        AccountTableWidget.setWindowTitle(_("Account table"))
        self.refreshButton.setText(_("Refresh"))
        self.filtersGroupBox.setTitle(_("Filters"))
        self.categoryLabel.setText(_("Category"))
        self.walletLabel.setText(_("Wallet"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import Optional

from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import QApplication, QDialog, QWidget

from tikka.domains.application import Application
from tikka.domains.entities.account import Account
from tikka.domains.entities.constants import DATA_PATH
from tikka.slots.pyqt.entities.constants import ADDRESS_MONOSPACE_FONT_NAME
from tikka.slots.pyqt.resources.gui.windows.account_rename_rc import (
    Ui_AccountRenameDialog,
)


class AccountRenameWindow(QDialog, Ui_AccountRenameDialog):
    """
    AccountRenameWindow class
    """

    def __init__(
        self,
        application: Application,
        account: Account,
        parent: Optional[QWidget] = None,
    ):
        """
        Init rename account window

        :param application: Application instance
        :param account: Account instance
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self.account = account
        self._ = self.application.translator.gettext

        # populate fields
        monospace_font = QFont(ADDRESS_MONOSPACE_FONT_NAME)
        monospace_font.setStyleHint(QFont.Monospace)
        self.addressValueLabel.setFont(monospace_font)
        self.addressValueLabel.setText(self.account.address)
        self.nameLineEdit.setText(self.account.name)

        # events
        self.buttonBox.accepted.connect(self.on_accepted_button)

    def on_accepted_button(self) -> None:
        """
        Triggered when user click on ok button

        :return:
        """
        self.account.name = self.nameLineEdit.text().strip()
        if self.account.name == "":
            self.account.name = None
        self.application.accounts.update(self.account)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    account_ = Account("CUYYUnh7N49WZhs5DULkmqw5Zu5fwsRBmE5LLrUFRpgw", name="old name")
    AccountRenameWindow(application_, account_).exec_()



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gql import gql

from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.indexer.accounts import (
    IndexerAccountsException,
    IndexerAccountsInterface,
)


class IndexerAccounts(IndexerAccountsInterface):
    """
    IndexerAccounts class
    """

    def is_legacy_v1(self, address: str) -> bool:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            IndexerAccountsInterface.is_legacy_v1.__doc__
        )
        if (
            not self.indexer.connection.is_connected()
            or self.indexer.connection.client is None
        ):
            raise IndexerAccountsException(NetworkConnectionError())

        query = gql(
            f"""
            query {{
                account (where: {{
                            createdOn: {{_eq: 0  }},
                            id: {{_eq: "{address}"}}
                        }}) {{
                            id
                        }}

            }}
            """
        )
        try:
            result = self.indexer.connection.client.execute(query)
        except Exception as exception:
            raise IndexerAccountsException(exception)

        #
        #    {
        #     "account": [
        #       {
        #         "id": "xxxxxxxx"
        #       }
        #     ]
        #    }
        #
        return len(result["account"]) > 0



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/wallet_save.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_WalletSaveDialog(object):
    def setupUi(self, WalletSaveDialog):
        WalletSaveDialog.setObjectName("WalletSaveDialog")
        WalletSaveDialog.resize(747, 355)
        self.verticalLayout = QtWidgets.QVBoxLayout(WalletSaveDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.imageLabel = QtWidgets.QLabel(WalletSaveDialog)
        self.imageLabel.setMaximumSize(QtCore.QSize(200, 200))
        self.imageLabel.setText("")
        self.imageLabel.setPixmap(QtGui.QPixmap(":/images/keys"))
        self.imageLabel.setScaledContents(True)
        self.imageLabel.setObjectName("imageLabel")
        self.verticalLayout.addWidget(self.imageLabel, 0, QtCore.Qt.AlignHCenter)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.addressLabel = QtWidgets.QLabel(WalletSaveDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setObjectName("addressLabel")
        self.gridLayout.addWidget(self.addressLabel, 0, 0, 1, 1)
        self.passwordLineEdit = QtWidgets.QLineEdit(WalletSaveDialog)
        self.passwordLineEdit.setText("")
        self.passwordLineEdit.setAlignment(QtCore.Qt.AlignCenter)
        self.passwordLineEdit.setReadOnly(True)
        self.passwordLineEdit.setObjectName("passwordLineEdit")
        self.gridLayout.addWidget(self.passwordLineEdit, 2, 1, 1, 1)
        self.fileFormatLabel = QtWidgets.QLabel(WalletSaveDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.fileFormatLabel.setFont(font)
        self.fileFormatLabel.setObjectName("fileFormatLabel")
        self.gridLayout.addWidget(self.fileFormatLabel, 1, 0, 1, 1)
        self.passwordLabel = QtWidgets.QLabel(WalletSaveDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.passwordLabel.setFont(font)
        self.passwordLabel.setObjectName("passwordLabel")
        self.gridLayout.addWidget(self.passwordLabel, 2, 0, 1, 1)
        self.addressValueLabel = QtWidgets.QLabel(WalletSaveDialog)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.gridLayout.addWidget(self.addressValueLabel, 0, 1, 1, 1)
        self.fileFormatValueLabel = QtWidgets.QLabel(WalletSaveDialog)
        self.fileFormatValueLabel.setObjectName("fileFormatValueLabel")
        self.gridLayout.addWidget(self.fileFormatValueLabel, 1, 1, 1, 1)
        self.changeButton = QtWidgets.QPushButton(WalletSaveDialog)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.changeButton.setIcon(icon)
        self.changeButton.setObjectName("changeButton")
        self.gridLayout.addWidget(self.changeButton, 2, 2, 1, 1, QtCore.Qt.AlignLeft)
        self.verticalLayout.addLayout(self.gridLayout)
        self.browseFilesButton = QtWidgets.QPushButton(WalletSaveDialog)
        self.browseFilesButton.setObjectName("browseFilesButton")
        self.verticalLayout.addWidget(self.browseFilesButton, 0, QtCore.Qt.AlignHCenter)
        self.errorLabel = QtWidgets.QLabel(WalletSaveDialog)
        self.errorLabel.setStyleSheet("color: red;")
        self.errorLabel.setText("")
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(WalletSaveDialog)
        QtCore.QMetaObject.connectSlotsByName(WalletSaveDialog)

    def retranslateUi(self, WalletSaveDialog):
        #  _translate = QtCore.QCoreApplication.translate
        WalletSaveDialog.setWindowTitle(_("Save wallet"))
        self.addressLabel.setText(_("Address"))
        self.fileFormatLabel.setText(_("File format"))
        self.passwordLabel.setText(_("Password"))
        self.fileFormatValueLabel.setText(_("EWIF"))
        self.changeButton.setText(_("Change"))
        self.browseFilesButton.setText(_("Browse files..."))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import logging
import struct
from typing import TYPE_CHECKING, Dict, List, Optional

from tikka.domains.entities.identity import Certification, Identity, IdentityStatus
from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.node.identities import (
    NodeIdentitiesException,
    NodeIdentitiesInterface,
)
from tikka.libs.keypair import Keypair, KeypairType

if TYPE_CHECKING:
    from tikka.adapters.network.node.node import NetworkNode


class NodeIdentities(NodeIdentitiesInterface):
    """
    NodeIdentities class
    """

    status_map = {
        "Unconfirmed": IdentityStatus.UNCONFIRMED,
        "Unvalidated": IdentityStatus.UNVALIDATED,
        "Member": IdentityStatus.MEMBER,
        "NotMember": IdentityStatus.NOT_MEMBER,
        "Revoked": IdentityStatus.REVOKED,
    }

    def __init__(self, node: "NetworkNode") -> None:
        """
        Use NetworkNodeInterface to request/send smiths information

        :param node: NetworkNodeInterface instance
        :return:
        """
        self.node = node

    def get_identity_index(self, address: str) -> Optional[int]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.get_identity_index.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        try:
            result = self.node.connection.client.query(
                "Identity", "IdentityIndexOf", [address]
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        return result

    def get_identity_indice(self, addresses: List[str]) -> List[Optional[int]]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.get_identity_indice.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        storage_functions = []
        for address in addresses:
            storage_functions.append(("Identity", "IdentityIndexOf", [address]))

        try:
            multi_result = self.node.connection.client.query_multi(storage_functions)
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        return multi_result

    def get_identity(self, address: str) -> Optional[Identity]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.get_identity.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        try:
            index = self.node.connection.client.query(
                "Identity", "IdentityIndexOf", [address]
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        if index is None:
            return None

        return self.get_identity_by_index(index)

    def get_identity_by_index(self, index: int) -> Optional[Identity]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.get_identity_by_index.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        try:
            result = self.node.connection.client.query(
                "Identity", "Identities", [index]
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        if result is None:
            return None

        old_address = result["old_owner_key"]
        if old_address is not None:
            old_address = old_address[0]

        # fixme: store next_creatable_on and removable_on as datetime in entity and DB
        return Identity(
            index=index,
            name=None,
            next_creatable_on=result["next_creatable_identity_on"],
            removable_on=int(result["next_scheduled"]),
            status=self.status_map[result["status"]],
            address=result["owner_key"],
            old_address=old_address,
            first_eligible_ud=result["data"]["first_eligible_ud"],
        )

    def get_identities(self, addresses: List[str]) -> Dict[str, Optional[Identity]]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.get_identities.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        identity_indice = self.get_identity_indice(addresses)
        index_by_address = dict(zip(addresses, identity_indice))
        filtered_indice = [index for index in identity_indice if index is not None]
        identity_by_index = self.get_identities_by_index(list(set(filtered_indice)))

        identity_by_address = {}
        for address, index in index_by_address.items():
            if index is not None:
                identity_by_address[address] = identity_by_index[index]
            else:
                identity_by_address[address] = None

        return identity_by_address

    def get_identities_by_index(
        self, identity_indice: List[int]
    ) -> Dict[int, Optional[Identity]]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.get_identities_by_index.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        storage_functions = []
        for identity_index in identity_indice:
            storage_functions.append(("Identity", "Identities", [identity_index]))

        try:
            multi_result = self.node.connection.client.query_multi(storage_functions)
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        # def: {
        #   Composite: {
        #     fields: [
        #       {
        #         name: data
        #         # type: 268
        #         typeName: IdtyData
        #         docs: []
        #       }
        #       {
        #         name: next_creatable_identity_on
        #         # type: 4
        #         typeName: BlockNumber
        #         docs: []
        #       }
        #       {
        #         name: old_owner_key
        #         # type: 269
        #         typeName: Option<(AccountId, BlockNumber)>
        #         docs: []
        #       }
        #       {
        #         name: owner_key
        #         # type: 0
        #         typeName: AccountId
        #         docs: []
        #       }
        #       {
        #         name: next_scheduled
        #         # type: 4
        #         typeName: BlockNumber
        #         docs: []
        #       }
        #       {
        #         name: status
        #         # type: 271
        #         typeName: IdtyStatus
        #         docs: []
        #       }
        #     ]
        #   }
        # }

        identities: Dict[int, Optional[Identity]] = {}
        for index, value_obj in enumerate(multi_result):
            if value_obj is not None:
                old_address_value = value_obj["old_owner_key"]
                old_address = None
                if old_address_value is not None:
                    if isinstance(old_address_value, tuple):
                        old_address = old_address_value[0]
                    else:
                        old_address = old_address_value

                identities[identity_indice[index]] = Identity(
                    index=identity_indice[index],
                    name=None,
                    next_creatable_on=value_obj["next_creatable_identity_on"],
                    removable_on=int(value_obj["next_scheduled"]),
                    status=self.status_map[value_obj["status"]],
                    address=value_obj["owner_key"],
                    old_address=old_address,
                    first_eligible_ud=value_obj["data"]["first_eligible_ud"],
                )
            else:
                identities[identity_indice[index]] = None

        return identities

    def change_owner_key(self, old_keypair: Keypair, new_keypair: Keypair) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.change_owner_key.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        identity_index = self.get_identity_index(old_keypair.ss58_address)
        if identity_index is None:
            raise NodeIdentitiesException("No identity found for origin account")

        # message to sign
        prefix_bytes = b"icok"
        genesis_hash_str = self.node.connection.client.get_block_hash(0)
        assert genesis_hash_str is not None
        genesis_hash_bytes = bytearray.fromhex(genesis_hash_str[2:])
        identity_index_bytes = struct.pack("<I", identity_index)
        identity_pubkey_bytes = old_keypair.public_key
        message_bytes = (
            prefix_bytes
            + genesis_hash_bytes
            + identity_index_bytes
            + identity_pubkey_bytes
        )

        # message signed by the new owner
        signature_bytes = new_keypair.sign(message_bytes)

        # newKey: AccountId32, newKeySig: SpRuntimeMultiSignature
        crypto_label = (
            "Sr25519" if new_keypair.crypto_type == KeypairType.SR25519 else "Ed25519"
        )
        params = {
            "new_key": new_keypair.ss58_address,
            "new_key_sig": {crypto_label: signature_bytes},
        }
        try:
            # create raw call (extrinsic)
            call = self.node.connection.client.compose_call(
                call_module="Identity",
                call_function="change_owner_key",
                call_params=params,
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        try:
            # create extrinsic signed by current owner
            extrinsic = self.node.connection.client.create_signed_extrinsic(
                call=call, keypair=old_keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        try:
            result = self.node.connection.client.submit_extrinsic(
                extrinsic, wait_for_inclusion=True
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        if not result.is_success:
            logging.error(result.error_message)
            message = (
                result.error_message.docs[0]
                if result.error_message
                else "identities.change_owner_key error"
            )
            raise NodeIdentitiesException(message)

    def certs_by_receiver(
        self, receiver_address: str, receiver_identity_index: int
    ) -> List[Certification]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.certs_by_receiver.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        try:
            result = self.node.connection.client.query(
                "Certification", "CertsByReceiver", [receiver_identity_index]
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        storage_functions = []
        for issuer_identity_index, cert_expire_on_block in result:
            storage_functions.append(
                ("Identity", "Identities", [issuer_identity_index])
            )

        multi_result = self.node.connection.client.query_multi(storage_functions)

        certifications = []
        for index, value_obj in enumerate(multi_result):
            certifications.append(
                Certification(
                    issuer_identity_index=result[index][0],
                    issuer_address=value_obj["owner_key"],
                    receiver_identity_index=receiver_identity_index,
                    receiver_address=receiver_address,
                    expire_on_block=result[index][1],
                )
            )

        return certifications

    def claim_uds(self, keypair: Keypair) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NodeIdentitiesInterface.claim_uds.__doc__
        )
        if (
            not self.node.connection.is_connected()
            or self.node.connection.client is None
        ):
            raise NodeIdentitiesException(NetworkConnectionError())

        try:
            # create raw call (extrinsic)
            call = self.node.connection.client.compose_call(
                call_module="UniversalDividend",
                call_function="claim_uds",
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        try:
            # create extrinsic signed by current owner
            extrinsic = self.node.connection.client.create_signed_extrinsic(
                call=call, keypair=keypair
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        try:
            result = self.node.connection.client.submit_extrinsic(
                extrinsic, wait_for_inclusion=True
            )
        except Exception as exception:
            logging.exception(exception)
            raise NodeIdentitiesException(exception)

        if not result.is_success:
            logging.error(result.error_message)
            message = (
                result.error_message.docs[0]
                if result.error_message
                else "identities.claim_uds error"
            )
            raise NodeIdentitiesException(message)
